import { ActionFormData, ModalFormData } from '@minecraft/server-ui';
import { world, system, EquipmentSlot } from '@minecraft/server';
let inventories = {};
const equipmentSlots = [EquipmentSlot.Head, EquipmentSlot.Chest, EquipmentSlot.Legs, EquipmentSlot.Feet, EquipmentSlot.Offhand];

system.beforeEvents.watchdogTerminate.subscribe(s => { s.cancel = true });

const playerInventories = () => system.run(() => {
  const players = world.getAllPlayers();
  if (players.length > 1) {
    updateInventory();
  }
  playerInventories();
});
playerInventories();

function getInventoryTag(player) {
  for (let tag of player.getTags()) {
    if (tag.startsWith("gwim:inventory:")) {
      return tag;
    }
  }
  return null;
}

function updateInventory() {
  const players = world.getAllPlayers();
  for (const player of players) {
    const playerTag = getInventoryTag(player);
    if (!playerTag) continue;
    const inventory = player.getComponent('inventory').container;
    const equipment = player.getComponent('minecraft:equippable');

    if (inventories[playerTag]) {
      const thisInventory = newInventory(inventory);
      const thisEquipment = newInventory(equipment, true);
      if (!matchedObject(inventories[playerTag].playerInventory, thisInventory) || !matchedObject(inventories[playerTag].playerEquipment, thisEquipment)) {
        players.forEach(f => {
          if (getInventoryTag(f) === playerTag && f.name !== player.name) {
            const container = f.getComponent('inventory').container;
            for (let i = 0; i < 36; i++) {
              const itemStack = inventory.getItem(i);
              container.setItem(i, itemStack);
            }
            const equip = f.getComponent('minecraft:equippable');
            for (const slot of equipmentSlots) {
              const itemStack = equipment.getEquipment(slot);
              equip.setEquipment(slot, itemStack);
            }
          }
        });
        inventories[playerTag] = { playerInventory: thisInventory, playerEquipment: thisEquipment };
      }
    } else {
      const otherPlayer = players.find(f => getInventoryTag(f) === playerTag && f.name !== player.name);
      if (otherPlayer) {
        const otherInventory = otherPlayer.getComponent('inventory');
        const otherEquipment = otherPlayer.getComponent('minecraft:equippable');
        for (let i = 0; i < 36; i++) {
          const itemStack = otherInventory.container.getItem(i);
          if (itemStack) inventory.setItem(i, itemStack);
        }
        for (const slot of equipmentSlots) {
          equipment.setEquipment(slot, otherEquipment.getEquipment(slot));
        }
        inventories[playerTag] = { playerInventory: newInventory(inventory), playerEquipment: newInventory(equipment, true) };
      }
    }
  }
}

function matchedObject(a, b) {
  if (a.length !== b.length) return false;
  return a.every(item => b.some(otherItem => JSON.stringify(item) === JSON.stringify(otherItem)));
}

function newInventory(container, equipment = false) {
  let items = [];
  if (!equipment) {
    for (let i = 0; i < 36; i++) {
      const itemStack = container.getItem(i);
      if (itemStack) {
        const durability = itemStack.getComponent("durability") ? itemStack.getComponent("durability").damage : 0;
        items.push({ typeId: itemStack.typeId, amount: itemStack.amount, durability: durability, slot: i });
      }
    }
  } else {
    for (const slot of equipmentSlots) {
      const slotItem = container.getEquipment(slot);
      const slotType = slotItem ? slotItem.typeId : '';
      const slotDurability = slotItem ? slotItem.getComponent("durability") ? slotItem.getComponent("durability").damage : 0 : 0;
      items.push({ typeId: slotType, durability: slotDurability, slot: slot });
    }
  }
  return items;
}

function showInventoryForm(player) {
  system.run(() => {
    const form = new ActionFormData()
      .title("Manage Shared Inventory")
      .button("Create Shared Inventory")
      .button("Remove Shared Inventory")
      .button("Cancel");

    form.show(player).then(response => {
      if (response.canceled) return;
      switch (response.selection) {
        case 0:
          createSharedInventoryForm(player);
          break;
        case 1:
          removeSharedInventoryForm(player);
          break;
        default:
          break;
      }
    }).catch(error => {
      player.sendMessage(`Error displaying the form: ${error}`);
    });
  });
}

function createSharedInventoryForm(player) {
  system.run(() => {
    const form = new ModalFormData()
      .title("Create Shared Inventory");

    const players = world.getAllPlayers();

    form.dropdown("Select a Player", players.map(p => p.name));

    form.textField("Enter Inventory Number", "Enter a number...");

    form.show(player).then(response => {
      if (response.canceled) return;

      const selectedPlayerName = players[response.formValues[0]].name;
      const inventoryNumber = response.formValues[1];

      const selectedPlayer = players.find(p => p.name === selectedPlayerName);

      if (selectedPlayer) {
        player.runCommandAsync(`tag "${selectedPlayer.name}" add gwim:inventory:${inventoryNumber}`)
          .then(() => {
            selectedPlayer.sendMessage(`You are now sharing inventory as gwim:inventory:${inventoryNumber}`);
          }).catch(error => {
            player.sendMessage(`Error creating shared inventory: ${error}`);
          });
      } else {
        player.sendMessage("Error: Player not found.");
      }
    }).catch(error => {
      player.sendMessage(`Error creating shared inventory: ${error}`);
    });
  });
}

function removeSharedInventoryForm(player) {
  system.run(() => {
    const form = new ActionFormData()
      .title("Remove Shared Inventory");

    const players = world.getAllPlayers();
    let inventoryGroups = {};

    players.forEach(p => {
      const tag = getInventoryTag(p);
      if (tag) {
        if (!inventoryGroups[tag]) {
          inventoryGroups[tag] = [];
        }
        inventoryGroups[tag].push(p.name);
      }
    });

    const inventoryTags = Object.keys(inventoryGroups);
    inventoryTags.forEach((tag, index) => {
      form.button(`Inventory Group: ${tag} - Members: ${inventoryGroups[tag].join(", ")}`);
    });

    form.show(player).then(response => {
      if (response.canceled) return;
      const selectedTag = inventoryTags[response.selection];
      const selectedPlayers = inventoryGroups[selectedTag];

      selectedPlayers.forEach(playerName => {
        const playerToRemove = players.find(p => p.name === playerName);
        playerToRemove.runCommandAsync(`tag @s remove ${selectedTag}`)
          .catch(error => {
            player.sendMessage(`Error removing inventory: ${error}`);
          });
        playerToRemove.sendMessage(`You are no longer sharing inventory with: ${selectedPlayers.join(", ")}`);
      });

      delete inventories[selectedTag];
    }).catch(error => {
      player.sendMessage(`Error removing shared inventory: ${error}`);
    });
  });
}

world.beforeEvents.itemUse.subscribe(({ source, itemStack }) => {
  if (itemStack.typeId === "gwim:inventory") {
    showInventoryForm(source);
  }
});
